﻿using System.Drawing;
using System.Drawing.Drawing2D;


namespace PluralRichPresence.Avatar
{

    public class ImageSplitter
    {
        public int Width;
        public int Height;
        private Image[] images;

        public ImageSplitter(int width, int height, Image[] images)
        {
            Width = width;
            Height = height;
            this.images = images;
        }

        public Bitmap Draw()
        {
            if (OperatingSystem.IsWindowsVersionAtLeast(6, 1))
            {
                Bitmap imageBuffer = new Bitmap(Width, Height);
                
                using (Graphics graphicsObject = Graphics.FromImage(imageBuffer))
                {
                    graphicsObject.Clear(Color.Black);

                    float partAngle = 360.0f / images.Length;
                    float currentAngle = 0;
                    foreach (Image image in images)
                    {
                        GraphicsPath path = new GraphicsPath();
                        path.AddPie(-Width / 2, -Height / 2, Width * 2, Height * 2, currentAngle, partAngle);
                        graphicsObject.SetClip(path);
                        graphicsObject.DrawImage(image, 0, 0, Width, Height);
                        currentAngle += partAngle;
                    }
                }
                return imageBuffer;
            }
            {
                throw new NotSupportedException("This OS is unsupported");
            }
        }
    }
}
