﻿namespace PluralRichPresence
{
    public static class Config
    {
        private const string CFG_FILE = "plurality.cfg";
        private const string SEPERATOR = ":";
        private static Dictionary<string, string> cfgEntries = new Dictionary<string, string>();

        static Config()
        {
            loadCfg();
        }

        private static void loadCfg()
        {
            cfgEntries.Clear();
            if (!File.Exists(CFG_FILE))
                saveCfg();

            using (StreamReader cfgReader = File.OpenText(CFG_FILE))
            {
                for (string? line = cfgReader.ReadLine(); line is not null; line = cfgReader.ReadLine())
                {
                    string[] values = line.Split(SEPERATOR);
                    cfgEntries.Add(values.First().Trim(), String.Join(SEPERATOR, values.Skip(1)).Trim());
                }

            }
        }
        private static void saveCfg()
        {
            using (StreamWriter cfgWriter = File.CreateText(CFG_FILE))
                foreach (KeyValuePair<String, String> values in cfgEntries)
                    cfgWriter.WriteLine(String.Join(SEPERATOR, new String[] { values.Key.Trim(), values.Value.Trim() }));    
        }
        public static void SetEntry(string key, string value)
        {
            key = key.Trim();
            value = value.Trim().ReplaceLineEndings(String.Empty);

            if (EntryExists(key))
                cfgEntries[key] = value;
            else
                cfgEntries.Add(key, value);

            saveCfg();
        }
        public static bool EntryExists(string key)
        {
            return cfgEntries.ContainsKey(key);
        }

        public static string GetEntry(string key)
        {
            return EntryExists(key) ? cfgEntries[key] : String.Empty;
        }
        public static void RemoveEntry(string key)
        {
            cfgEntries.Remove(key.Trim());
            saveCfg();
        }
        public static string GetEntry(string key, string defaultValue)
        {
            if (!EntryExists(key))
            {
                SetEntry(key, defaultValue);
                return defaultValue;
            }
            return GetEntry(key);
        }

    }
}
